#include <bickley/bkl.h>

#include "hrn.h"
#include "hrn-iface-player.h"
#include "hrn-image-player.h"

enum
{
  PROP_0,
};

struct _HrnImagePlayerPrivate
{
  ClutterActor *image_player;
  guint32       progress_id;
};

static void hrn_player_init (HrnIfacePlayerClass *iface);
#define GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
                                                          HRN_TYPE_IMAGE_PLAYER, \
                                                          HrnImagePlayerPrivate))
G_DEFINE_TYPE_WITH_CODE (HrnImagePlayer, hrn_image_player, CLUTTER_TYPE_ACTOR,
                         G_IMPLEMENT_INTERFACE (HRN_TYPE_IFACE_PLAYER,
                                                hrn_player_init));

static void
hrn_image_player_finalize (GObject *object)
{
  HrnImagePlayer        *player = (HrnImagePlayer *) object;
  HrnImagePlayerPrivate *priv   = player->priv;

  if (priv->progress_id > 0)
    {
      g_source_remove (priv->progress_id);
      priv->progress_id = 0;
    }

  G_OBJECT_CLASS (hrn_image_player_parent_class)->finalize (object);
}

static void
hrn_image_player_dispose (GObject *object)
{
  G_OBJECT_CLASS (hrn_image_player_parent_class)->dispose (object);
}

static void
hrn_image_player_set_property (GObject *object, guint prop_id,
                               const GValue *value,
                               GParamSpec   *pspec)
{
  switch (prop_id)
    {
      default:
        break;
    }
}

static void
hrn_image_player_get_property (GObject *object, guint prop_id,
                               GValue     *value,
                               GParamSpec *pspec)
{
  switch (prop_id)
    {
      default:
        break;
    }
}

static void
hrn_image_player_allocate (ClutterActor *actor, const ClutterActorBox *box,
                           ClutterAllocationFlags flags)
{
  HrnImagePlayer        *player = (HrnImagePlayer *) actor;
  HrnImagePlayerPrivate *priv   = player->priv;

  CLUTTER_ACTOR_CLASS (hrn_image_player_parent_class)->allocate
                             (actor, box, flags);

  if (priv->image_player)
    {
      clutter_actor_allocate_preferred_size (priv->image_player,
                                             flags);
    }
}

static void
hrn_image_player_paint (ClutterActor *actor)
{
  HrnImagePlayer        *player = (HrnImagePlayer *) actor;
  HrnImagePlayerPrivate *priv   = player->priv;

  if (priv->image_player && CLUTTER_ACTOR_IS_VISIBLE (priv->image_player))
    {
      clutter_actor_paint (priv->image_player);
    }
}

static void
hrn_image_player_pick (ClutterActor *actor, const ClutterColor *color)
{
  hrn_image_player_paint (actor);
}

static void
hrn_image_player_map (ClutterActor *self)
{
  HrnImagePlayerPrivate *priv = HRN_IMAGE_PLAYER (self)->priv;

  CLUTTER_ACTOR_CLASS (hrn_image_player_parent_class)->map (self);

  if (priv->image_player)
    clutter_actor_map (CLUTTER_ACTOR (priv->image_player));
}

static void
hrn_image_player_unmap (ClutterActor *self)
{
  HrnImagePlayerPrivate *priv = HRN_IMAGE_PLAYER (self)->priv;

  CLUTTER_ACTOR_CLASS (hrn_image_player_parent_class)->unmap (self);

  if (priv->image_player)
    clutter_actor_unmap (CLUTTER_ACTOR (priv->image_player));
}


static void
hrn_image_player_class_init (HrnImagePlayerClass *klass)
{
  GObjectClass      *o_class = (GObjectClass *) klass;
  ClutterActorClass *a_class = (ClutterActorClass *) klass;

  o_class->dispose      = hrn_image_player_dispose;
  o_class->finalize     = hrn_image_player_finalize;
  o_class->set_property = hrn_image_player_set_property;
  o_class->get_property = hrn_image_player_get_property;

  a_class->allocate = hrn_image_player_allocate;
  a_class->paint    = hrn_image_player_paint;
  a_class->pick     = hrn_image_player_pick;
  a_class->map      = hrn_image_player_map;
  a_class->unmap    = hrn_image_player_unmap;

  g_type_class_add_private (klass, sizeof (HrnImagePlayerPrivate));
}

static gboolean
image_display_finished (gpointer data)
{
  HrnImagePlayer        *image_player = (HrnImagePlayer *) data;
  HrnImagePlayerPrivate *priv         = image_player->priv;

  g_debug ("Image display finished");
  hrn_iface_player_emit_eos ((HrnIfacePlayer *) image_player);

  priv->progress_id = 0;
  return FALSE;
}

static void
player_set_playing (HrnIfacePlayer *player, gboolean playing)
{
  HrnImagePlayer        *image = (HrnImagePlayer *) player;
  HrnImagePlayerPrivate *priv  = image->priv;

  if (playing)
    {
      if (priv->progress_id == 0)
        {
          priv->progress_id = g_timeout_add_seconds (5,
                                                     image_display_finished,
                                                     image);
        }
    }
  else
    {
      if (priv->progress_id > 0)
        {
          g_source_remove (priv->progress_id);
          priv->progress_id = 0;
        }
    }
}


static void
player_set_uri (HrnIfacePlayer *player, const char     *uri)
{
  HrnImagePlayer        *image = (HrnImagePlayer *) player;
  HrnImagePlayerPrivate *priv  = image->priv;
  BklItem               *item;
  const gchar           *orient;
  gfloat                 w, h;
  gdouble                xs, ys, scale;
  gint                   x, y, sw, sh;

  if (uri == NULL)
    {
      g_debug ("Image player got NULL URI");
      /* FIXME: Should blank the image? */
      return;
    }

  item = hrn_get_item_for_uri (uri);
  if (item == NULL)
    {
        g_debug ("showing image not in bickley dbs %s", uri);
    }

  /* handle http / UPNP directly with gdkpixbuf and GIO */
  if (g_str_has_prefix (uri, "http"))
    {
      GdkPixbuf *pixbuf = NULL;
      GFile *file;
      GFileInputStream *stream;
      GError *error = NULL;

      file = g_file_new_for_uri (uri);
      stream = g_file_read (file, NULL, NULL);
      if (stream)
        {
          pixbuf = gdk_pixbuf_new_from_stream (G_INPUT_STREAM (stream), NULL, NULL);
          g_object_unref (stream);
          g_object_unref (file);

          if (pixbuf)
            {
              clutter_texture_set_from_rgb_data ((ClutterTexture *) priv->image_player,
                                              gdk_pixbuf_get_pixels (pixbuf),
                                              gdk_pixbuf_get_has_alpha (pixbuf),
                                              gdk_pixbuf_get_width (pixbuf),
                                              gdk_pixbuf_get_height (pixbuf),
                                              gdk_pixbuf_get_rowstride (pixbuf),
                                              gdk_pixbuf_get_has_alpha (pixbuf) ? 4 : 3,
                                              0,
                                              &error);
              if (error)
                {
                  g_warning ("Unable to set pixbuf : %s", error->message);
                  g_error_free (error);
                }
              g_object_unref (pixbuf);
            }
        }

      if (pixbuf == NULL)
        {
          guchar px[8]="";
          /* we get a crash if we set no data at all */
          clutter_texture_set_from_rgb_data ((ClutterTexture *) priv->image_player, px, FALSE,
                                              1, 1, 3, 3, 0, NULL);
          return;
        }
    }
  else
    {
      GFile *file;
      gchar *path;
      file = g_file_new_for_commandline_arg (uri);
      path = g_file_get_path (file);

      if (path == NULL)
        {
          g_object_unref (file);
          return;
        }

      if (path)
        {
          clutter_texture_set_from_file ((ClutterTexture *) priv->image_player,
                                         path, NULL);
        }
       g_object_unref (file);
    }

  if (item)
    {
      orient = bkl_item_image_get_orientation (BKL_ITEM_IMAGE (item));
      if (orient && g_str_equal (orient, "right - top"))
        {
          /* FIXME: deal with rotation */
        }
    }

  clutter_actor_get_size (priv->image_player, &w, &h);
  gtk_window_get_size (GTK_WINDOW (window), &sw, &sh);

  xs = ((double) sw) / (double) w;
  ys = ((double) sh) / (double) h;

  if (xs > ys)
    {
      x     = (((float) sw) - (w * ys)) / 2;
      y     = 0;
      scale = ys;
    }
  else
    {
      x     = 0;
      y     = (((float ) sh) - (h * xs)) / 2;
      scale = xs;
    }

  clutter_actor_set_position (priv->image_player, x, y);
  clutter_actor_set_scale (priv->image_player, scale, scale);
}

static void
player_set_position (HrnIfacePlayer *player, double position)
{
  /* This function intentionally left blank */
}

static void
hrn_player_init (HrnIfacePlayerClass *iface)
{
  iface->set_playing  = player_set_playing;
  iface->set_uri      = player_set_uri;
  iface->set_position = player_set_position;
}

static void
hrn_image_player_init (HrnImagePlayer *self)
{
  HrnImagePlayerPrivate *priv = GET_PRIVATE (self);

  self->priv = priv;

  priv->image_player = clutter_texture_new ();
  clutter_actor_set_reactive (priv->image_player, TRUE);
  clutter_texture_set_filter_quality (CLUTTER_TEXTURE (priv->image_player),
                                      CLUTTER_TEXTURE_QUALITY_HIGH);
  clutter_actor_set_parent (priv->image_player, CLUTTER_ACTOR (self));
  clutter_actor_show (priv->image_player);
}

